process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
process.env.SUPPRESS_NO_CONFIG_WARNING = true

const httpMocks = require('node-mocks-http')
const fixtures = require('./../fixtures/fixtures')
const Model = require('./../helpers/Model')

const models = Model.build()
jest.mock('pubsweet-component-mail-service', () => ({
  setupAssignEmail: jest.fn(),
  setupHandlingEditorAgreedEmail: jest.fn(),
  setupDeclineEmail: jest.fn(),
}))

const notFoundError = new Error()
notFoundError.name = 'NotFoundError'
notFoundError.status = 404

const { handlingEditor, invitedHandlingEditor, author } = fixtures.users
const { standardCollection } = fixtures.collections
const { heTeam } = fixtures.teams
const body = {
  isAccepted: true,
}
const patchPath = '../../routes/collectionsInvitations/patch'
describe('Patch collections invitations route handler', () => {
  it('should return success when the handling editor accepts work on a collection', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = handlingEditor.id
    req.params.collectionId = standardCollection.id
    req.params.invitationId = standardCollection.invitations[0].id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)
    expect(res.statusCode).toBe(200)
    expect(standardCollection.invitations[0].hasAnswer).toBeTruthy()
    expect(standardCollection.invitations[0].isAccepted).toBeTruthy()
  })
  it('should return success when the handling editor declines work on a collection', async () => {
    body.isAccepted = false
    const req = httpMocks.createRequest({
      body,
    })
    req.user = invitedHandlingEditor.id
    req.params.collectionId = standardCollection.id
    req.params.invitationId = standardCollection.invitations[1].id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(200)
    expect(standardCollection.invitations[1].hasAnswer).toBeTruthy()
    expect(standardCollection.invitations[1].isAccepted).toBeFalsy()
    expect(heTeam.members).not.toContain(invitedHandlingEditor.id)
    expect(invitedHandlingEditor.teams).not.toContain(heTeam.id)
  })
  it('should return an error params are missing', async () => {
    delete body.isAccepted
    const req = httpMocks.createRequest({
      body,
    })
    req.user = handlingEditor.id
    req.params.collectionId = standardCollection.id
    req.params.invitationId = standardCollection.invitations[0].id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Missing parameters')
    body.isAccepted = true
  })
  it('should return an error if the collection does not exists', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = handlingEditor.id
    req.params.collectionId = 'invalid-id'
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(404)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('collection not found')
  })
  it('should return an error when the invitation does not exist', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = author.id
    req.params.collectionId = standardCollection.id
    req.params.invitationId = 'invalid-id'
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(404)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Invitation invalid-id not found')
  })
})
